package miniteller;

import org.omg.CORBA.FloatHolder;

public class AccountServ extends _AccountImplBase {

  private static final int FIRST_MAX = 20, LAST_MAX = 25;

	// dependent on properties of a long - this is pretty bad style
	// although, being Java, I know (hope?) there won't be platform
	// variation here
  private static final double AMT_MAX = 10000000;

  int acctno;
  private MiniTellerServ mts;
  protected float bal;
  String firstName, lastName;

  public AccountServ(String name) {
    super(name);
  }

  public AccountServ() {
    super();
  }

  // account-opening constructor
  AccountServ(MiniTellerServ teller, int acctno, float bal) throws DataInvalid {
    super();
    setOpenInfo(teller, acctno, bal);
  }

  // data-reanimation constructor
  AccountServ(MiniTellerServ teller, int acctno, String firstName,
	      String lastName, float bal) {
    super();
    try {
      setOpenInfo(teller, acctno, bal); // can't do this() because of exception
      if (firstName == null) firstName = "";
      if (lastName == null) lastName = "";
      setName(firstName, lastName);
    } catch (DataInvalid e) { // this shouldn't happen
      System.err.println("Information retrieved from database violates hard limits:");
      System.err.println("   " + bal + "; " + firstName + " " + lastName);
    }
  }

  protected void setOpenInfo(MiniTellerServ teller, int acctno, float bal)
   throws DataInvalid {
    this.mts = teller;
    this.acctno = acctno;
         
    this.bal = round(bal);
  }

  public String[] getBasicInfo() throws AccessViolation {
    String[] bi = new String[3];
    try {
    bi[bidx_acct] = new StringBuffer(6).append(acctno).toString();
    bi[bidx_name] = formatName();
    bi[bidx_bal] = formatBal();
    } catch (Exception e) {
      System.err.println("Error in getting basic account information:");
      e.printStackTrace();
    }
    return bi;
  }

  public synchronized float getBalance() throws AccessViolation {
    return bal;
  }

  public synchronized float deposit(float amt)
   throws DataInvalid, AccessViolation {
    amt = round(amt);
    mts.setBal(this, round(bal + amt));
    bal += amt;
    return bal;
  }

  public synchronized float withdraw(float amt)
   throws DataInvalid, AccessViolation, InsufficientFunds {
    amt = round(amt);
    if (amt > bal) throw new InsufficientFunds(
     "Insufficient funds present in account to complete withdrawal.");
    mts.setBal(this, round(bal - amt));
    bal -= amt;
    return bal;
  }

  public synchronized float transfer(float amt, Account dest,
				     FloatHolder destbal)
   throws DataInvalid, AccessViolation, InsufficientFunds {
    if (dest == this) throw new AccessViolation(
      "Transferring from an account to the same account has no useful purpose.");
    amt = round(amt);
    if (amt > bal) throw new InsufficientFunds(
     "Insufficient funds present in source account to perform transfer.");
    synchronized (mts) {
      try {
      mts.beginTransaction();
      mts.setBal(this, round(bal - amt));
      mts.setBal(dest, round(bal + amt));
      mts.commitTransaction();
      } catch (AccessViolation e) {
        mts.rollbackTransaction();
        throw e;
      }
    }
    bal -= amt;
    destbal.value = (((AccountServ) dest).bal += amt);
    return bal;
  }

  protected static float round(double amt) throws DataInvalid {
    if (amt < 0) throw new DataInvalid("Amount must be nonnegative.");
		if (amt == Float.POSITIVE_INFINITY) throw new DataInvalid(
			"Amount must not be infinite.");
    if (amt >= AMT_MAX) throw new DataInvalid("Amount exceeds system limits.");
    return (float)((long)(amt * 100))/100;
  }

  public String[] getCustomerInfo() throws AccessViolation {
    CustInfo i = mts.getCustInfo(this);
    String[] s = new String[6];
    s[cidx_first] = firstName;
    s[cidx_last] = lastName;
    s[cidx_addr1] = i.addr1;
    s[cidx_addr2] = i.addr2;
    s[cidx_phone] = i.phone;
    s[cidx_ssn] = i.getFormattedSSN();
    return s;
  }

  public void setCustomerInfo(String firstName, String lastName,
			      String addr1, String addr2, String phone,
			      String ssn) throws DataInvalid, AccessViolation {
    CustInfo i;
    try {
    setName(firstName, lastName);
    i = new CustInfo(addr1, addr2, phone, ssn);
    } catch (DataInvalid e) { throw e; }
    try { mts.setCustInfo(this, i); }
    catch (AccessViolation e) { throw e; }
    catch (Exception e) {
      System.err.println("Internal error setting customer information: ");
      e.printStackTrace();
    }
  }

  void setName(String firstName, String lastName) throws DataInvalid {
    if (firstName.length() > FIRST_MAX)
      throw new DataInvalid("first name", FIRST_MAX);
    if (lastName.length() > LAST_MAX)
      throw new DataInvalid("last name", LAST_MAX);
    this.firstName = firstName; this.lastName = lastName;
  }

  public String formatName() {
    StringBuffer sb = new StringBuffer(FIRST_MAX + LAST_MAX + 2);
    sb.append(lastName);
    if (firstName != null && lastName != null) sb.append(", ");
    sb.append(firstName);

    return sb.toString();
  }

  public String formatBal() {
    StringBuffer sb = new StringBuffer(15);
    String bls = Long.toString(Math.round(bal * 100));
    int i = bls.length();
    char[] bla = new char[i];
    bls.getChars(0, i, bla, 0);
    sb.append('$');
    for (i = 0 ; i < (bla.length - 2) ; i++) {
      // *** commas here, eventually, maybe...
      sb.append(bla[i]);
    }
    if (i == 0) { sb.append('0'); }
    sb.append('.');
    if (bla.length == 1) sb.append('0'); else sb.append(bla[i++]);
    sb.append(bla[i]);
    return sb.toString();
  }

  public int acctno() {
    return acctno;
  }
}
