package miniteller;

import org.omg.CORBA.*;
import java.util.Properties;
import java.io.*;

public class MTServer {

  private static final String PROPS_FILENAME = "miniteller.properties";
  private static final String MINITELLER_DFLTID = "MiniTeller";
  private static final String ORB_ADDR = "trajan.invantage.com";
  private static final String ORB_PORT = "31453";

  static MiniTellerServ mt;

  public static void main(String args[]) {
    // Merge properties in file with system properties
    Properties props = new Properties(System.getProperties());
    try {
    props.load(new BufferedInputStream(new FileInputStream(PROPS_FILENAME)));
    System.setProperties(props);
    System.out.println("Read properties from file " + PROPS_FILENAME);
    } catch (IOException ie) {
      System.out.println("Proceeding with no properties file.");
    } // don't worry if the properties file can't be found

    // Do the CORBA dance and create the initial server-side object
    try {
    Properties orbprops = new Properties();
    orbprops.put("ORBnullString", "true");
    // others can go into system property list, they'll be picked up
    // in future
    props.put("ORBagentAddr", System.getProperty("ORBagentAddr", ORB_ADDR));
    props.put("ORBagentPort", System.getProperty("ORBagentPort", ORB_PORT));
    ORB orb = ORB.init(args, orbprops);
    BOA boa = orb.BOA_init();
    System.out.println("BOA inited");
    mt = new MiniTellerServ(
             props.getProperty("MTServer.tellername", MINITELLER_DFLTID));
    System.out.println("MiniTeller created");
    boa.obj_is_ready(mt);
    System.out.println("Registered object with BOA");
    boa.impl_is_ready();
    } catch (SystemException e) {
      System.err.println("Error starting server:\n\n" + e.getMessage());
    }
  }

}
